% Created 2012-August-20
% Written by Fu Lin
% Greedy algorithm for the noise-free leader selection problem
% Inputs: 
% L -> the Laplacian matrix of the graph
% k -> the number of leaders
% Outputs:
% Jval         -> the objective function values for the selection of leaders from 1
%                 up to k
% leader_idx   -> the indices of the leaders

function [Jval,leader_idx] = greedy_noisefree_leader(L,k)

n = size(L,1);
Jval = zeros(k,1);

% start with no leaders
leader_idx = [];

for i = 1:k
    
    i %#ok<NOPRT>

    % find the indices of the followers so far
    v = zeros(n,1);
    v(leader_idx) = 1;
    follower_idx = find(v~=1);
    
    % store the objective function for different selections of leaders
    Jtemp = zeros(n-i+1,1);
    
    for ii = 1:(n-i+1)
                        
        % add another leader from the followers to form the temporary leader set
        % there are i leaders and n-i followers so far
        leader_idx_temp = [leader_idx; follower_idx(ii)];
        
        % find the indices of the followers
        v = zeros(n,1);
        v(leader_idx_temp) = 1;
        follower_idx_temp = find(v~=1);
        
        % the reduced Laplacian matrix obtained from removing the columns
        % and rows corresponding to the indices of leaders        
        Lff = L(follower_idx_temp,follower_idx_temp);
        
        Jtemp(ii) = trace(inv(Lff));
    end
    
    % find the minimum of Jtemp and the index of the follower
    [Jval(i), new_leader] = min(Jtemp);
    
    % update the set of leaders
    leader_idx = [leader_idx; follower_idx(new_leader)]; %#ok<AGROW>
    
end