% Created 2012-Feb-9
% Written by Fu Lin
% Leader selection problem using the greedy algorithm.
% Input: L      ->    Laplacian matrix
%        k      ->    number of leaders
%        Da     ->    diagonal matrix with positive diagonal elements
% Output: Jopt        ->   objective function 
%         leader_idx  ->   indices of leaders
%         invL        ->   inverse of Laplacian with selected leaders

function [Jopt,leader_idx,invL] = greedyalg(L,k,Da)

Delta = Da;

n = size(L,1);
leaders = zeros(1,n);
followers = ones(1,n);

% inverse of L with the first node chosen as the leader

Lbar = L;
Lbar(1,1) = L(1,1) + Delta(1,1);
invLbar = sparse(inv(Lbar));
trinvLbar = trace(invLbar);
J = zeros(k,n);
J(1,1) = trinvLbar;
Jhis = zeros(1,k);

% sweep through the other nodes (2 to n) 
for i = 2:n      
    
    M1 = [-Delta(1,1)*invLbar(1,1), Delta(i,i) * invLbar(1,i);...
        -Delta(1,1)*invLbar(i,1) , Delta(i,i) * invLbar(i,i)];
    M2 = [ -Delta(1,1)*invLbar(1,:)*invLbar(:,1), Delta(i,i)*invLbar(1,:)*invLbar(:,i) ; ...
        -Delta(1,1)*invLbar(i,:)*invLbar(:,1), Delta(i,i)*invLbar(i,:)*invLbar(:,i) ];
    J(1,i) = J(1,1) - trace( inv(eye(2) + M1) * M2 );   %#ok<*MINV>
        
end

% update the leader and follower sets

[minJ,idx] = min(J(1,:));
Jhis(1) = minJ;
leaders(idx) = 1;
followers(idx) = 0;
ei = zeros(n,1);
ei(idx) = 1;
Lbar = L;
Lbar(idx,idx) = L(idx,idx) + Delta(idx,idx);
invL = inv(Lbar);

for l = 2:k
    
    trinvL = trace(invL);
    
    % we compute all n cases for each J
    % we then set those entries that are already selected-leaders to be
    % large values
    
    for i = 1:n     
        J(l,i) = trinvL - Delta(i,i)*norm(invL(:,i),2)^2/(1 + Delta(i,i)*invL(i,i));        
    end
    leader_idx = (leaders == 1);
    J(l,leader_idx) = 1.e+15;
    
    % we now take the minimum value of these J's and assign that node to be
    % the leader 
    
    [minJ,idx] = min(J(l,:));
    Jhis(l) = minJ;
    leaders(idx) = 1;
    followers(idx) = 0;

    % we need to update the inverse of L       
    
    invL = invL - Delta(idx,idx)*invL(:,idx)*invL(:,idx)'/(1 + Delta(idx,idx)*invL(idx,idx));
    
end

leader_idx = find(leaders==1);
Jopt = Jhis(k);
