% Created 2013-April-17
% Written by Fu Lin
% Greedy 2-swap algorithm to improve performance of a selection of
% noise-free leaders.
% Input: L           ->   Laplacian matrix
%        leader_idx  ->   index set of leaders
%        follower_idx->   index set of followers
%        Max_Iter    ->   maximum number of swaps
% Output: Jopt       ->   objective function 
%         leader_idx ->   index set of leaders
%         swap_step  ->   number of swaps

function [J,leader_idx,swap_step] = noisefree_twoswap_sorted(L,leader_idx,follower_idx,Max_Iter)

% number of leaders
k = length(leader_idx);

L = full(L);
n = size(L,1);

% the number of maximum swaps
Nmax = Max_Iter;

% current value of the objective function
v = zeros(n,1);
v(leader_idx) = 1;
one = ones(n,1);
x = one - v;
J = trace(pinv(L.*(x*x')));

for swap_step = 1:Nmax

    swap_step %#ok<NOPRT>
    swap = 0;
    
    for ii = 1:k

        i = leader_idx(ii);
        
        for jj = 1:(n-k)
            
            j = follower_idx(jj);
            
            vtemp = v;
            vtemp(i) = 0;
            vtemp(j) = 1;
            xtemp = one - vtemp;
            
            Jswap = trace(inv(L(find(xtemp),find(xtemp)))); %#ok<FNDSB>
            if Jswap < J                
                leader_idx(ii) = j;
                follower_idx(jj) = i;
                
                v = vtemp;
                J = Jswap;
                
                swap = 1;                               
            end
            
            if swap == 1
                break;
            end
            
        end        
        if swap == 1
            break;
        end        
    end         
            
        % After checking all k leaders and n-k followers swaps, and no
        % improvement can be found, 
        % we stop the algorithm
        
    if (ii == k) && (jj == n-k)
        break;
    end

end

if swap_step == Nmax
    disp('The number of maximum swaps is reached in swap algorithm!')
end