% Created 2012-Feb-9
% Written by Fu Lin
% A greedy 2-swap algorithm to improve the performance of a selection of
% leaders.
% Input: L           ->    Laplacian matrix
%        leader_idx  ->    index set of leaders
%        Da          ->    absolute feedback gain matrix
% Output: Jopt       ->   objective function 
%         leader_idx ->   index set of leaders
%         l          ->   number of swaps

function [Jopt,leader_idx,l] = twoswap(L,leader_idx,Da,Max_Iter)

% number of leaders
k = length(leader_idx);
n = size(L,1);
v = zeros(n,1);
v(leader_idx) = 1;

Delta = Da;

% the number of maximum swaps
% Nmax = 1*size(L,1);
Nmax = Max_Iter;

invL = inv( L + Delta * diag(v) );

% current value of the performance
Jc = trace(invL);
leader_idx = find(v==1)';
follower_idx = find(v==0)';

for l = 1:Nmax
    
    swap = 0;
    
    for ii = 1:k

        i = leader_idx(ii);
        
        for jj = 1:(n-k)
            
            j = follower_idx(jj);
            
            % use rank-2 update to compute the H2 norm without inverting
            % the matrix
            
            M1 = [-Delta(i,i)*invL(i,i), Delta(j,j) * invL(i,j);...
                -Delta(i,i)*invL(j,i) , Delta(j,j) * invL(j,j)];
            M2 = [ -Delta(i,i)*invL(i,:)*invL(:,i), Delta(j,j)*invL(i,:)*invL(:,j) ; ...
                -Delta(i,i)*invL(j,:)*invL(:,i), Delta(j,j)*invL(j,:)*invL(:,j) ];
            Jswap = Jc - trace( inv(eye(2) + M1) * M2 );  %#ok<*MINV>
        
            if Jswap < Jc
                
                leader_idx(ii) = j;
                follower_idx(jj) = i;
                
                v = zeros(n,1);
                v(leader_idx(1:k)) = 1;
                invL = invL - [ -Delta(i,i)*invL(:,i), Delta(j,j)*invL(:,j)]*...
                    inv(eye(2) + M1) * [invL(i,:); invL(j,:)];                
                Jc = Jswap;
                
                swap = 1;
                               
            end
            
            if swap == 1
                break;
            end
            
        end
        
        if swap == 1
            break;
        end
        
    end         
            
        % After checking all k leaders and n-k followers swaps, and no
        % improvement can be found, 
        % we stop the algorithm
        
    if (ii == k) && (jj == n-k)
        break;
    end

end

if l == Nmax
    disp('The number of maximum swaps is reached in swap algorithm!')
end

Jopt = Jc;
